#include <iostream>
#include <map>
#include <queue>
#include <vector>
#include <climits>

using namespace std;

#define x(index) ((index) % width)
#define y(index) ((index) / width)

int height = 0;
int width = 0;
int min_height = 0;
int max_height = 0;
int min_width = 0;
int max_width = 0;

void mark_distances(int start, vector<int>& cells) {
    map<int, bool> visited{};
    queue<pair<int, int>> to_visit{};

    visited[start] = true;
    to_visit.push({ start, 0 });

    auto handle_neighbour = [](int neighbour, int distance, map<int, bool>& visited, queue<pair<int, int>>& to_visit, vector<int>& cells) {
        if (visited[neighbour] || (cells[neighbour] & 1) == 0) {
            return;
        }

        int prev_dist = cells[neighbour] >> 1;
        cells[neighbour] = (max(prev_dist, distance + 1) << 1) + 1;

        visited[neighbour] = true;
        to_visit.push({ neighbour, distance + 1 });
    };

    while (!to_visit.empty()) {
        auto [current, distance] = to_visit.front();
        to_visit.pop();

        if (x(current) < max_width - 1) {
            handle_neighbour(current + 1, distance, visited, to_visit, cells);
        }

        if (x(current) > min_width) {
            handle_neighbour(current - 1, distance, visited, to_visit, cells);
        }

        if (y(current) < max_height - 1) {
            handle_neighbour(current + width, distance, visited, to_visit, cells);
        }

        if (y(current) > min_height) {
            handle_neighbour(current - width, distance, visited, to_visit, cells);
        }
    }
}

int main() {
    cin >> height >> width;

    min_width = width;
    min_height = height;

    vector<int> cells(height * width, 0);
    vector<int> holes{};

    for (int i = 0; i < height; i++) {
        for (int j = 0; j < width; j++) {
            char c;
            cin >> c;

            cells[i * width + j] = (c == 'F' ? 0 : 1);

            if (c == 'H') {
                holes.push_back(i * width + j);

                min_width = min(min_width, j);
                min_height = min(min_height, i);
                max_width = max(max_width, j + 1);
                max_height = max(max_height, i + 1);
            }
        }
    }

    for (int hole : holes) {
        mark_distances(hole, cells);
    }

    // for (int i = 0; i < height; i++) {
    //     for (int j = 0; j < width; j++) {
    //         cout << cells[i * width + j] << " ";
    //     }
    //     cout << "\n";
    // }

    vector<int> answer{};
    int min_worst_dist = INT_MAX;

    for (int i = 0; i < width * height; i++) {
        if ((cells[i] & 1) == 0) {
            continue;
        }

        int dist = cells[i] >> 1;

        if (dist < min_worst_dist) {
            min_worst_dist = dist;
            answer.clear();
            answer.push_back(i);
        } else if (dist == min_worst_dist) {
            answer.push_back(i);
        }
    }

    cout << answer.size() << "\n";

    for (int index : answer) {
        cout << (y(index) + 1) << " " << (x(index) + 1) << "\n";
    }

    return 0;
}
